package puppet.ambition;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.wsdl.Binding;
import javax.wsdl.Definition;
import javax.wsdl.Operation;
import javax.wsdl.PortType;
import javax.wsdl.WSDLException;


import org.antlr.stringtemplate.StringTemplate;

import com.ibm.wsdl.xml.WSDLReaderImpl;


public class ClassCodeProcessor {

    private String name;
    
//    private ArrayList<MethodCodeProcessor> methodList;
    
    private String targetMethodsPrefix;

//  Are the qualificator such as "private", "public" and so on    
    private String classPrefix;
    
    private String WSDLPath;
    
    private PatternsReader ambitionPatterns;
    
    private AmbitionVariableWrapper ambitionVarWrapper;
    
    public ClassCodeProcessor (String className, String WSDLPath, String targetMethodsPrefix){
        this.name = className;
        this.WSDLPath = WSDLPath;
        this.targetMethodsPrefix = targetMethodsPrefix;

        this.ambitionPatterns = new PatternsReader();
        this.ambitionVarWrapper = new AmbitionVariableWrapper();
//        methodList =  new ArrayList<MethodCodeProcessor>();
    }
    
    public ClassCodeProcessor (String className, String WSDLPath){
        this(className,WSDLPath,"public");
    }

    
// Note that the method type it will not be set!! 
    public ArrayList<MethodCodeProcessor> extractMethodsNameOnly (String code){
        WSDLReaderImpl wsdlService = new WSDLReaderImpl();
        Definition wsdlDocument = null;
        TypeConverter converter = new TypeConverter();

        try {
            wsdlDocument = wsdlService.readWSDL(this.WSDLPath);
        } catch (WSDLException e) {
            e.printStackTrace();
        }
        
        ArrayList<MethodCodeProcessor> methodList = new ArrayList<MethodCodeProcessor>();
        String methodName;
        String methodType;
        boolean foundIt = false;

        String bindingName = this.name.replaceFirst("BindingImpl", "Binding");
        
        System.out.println(this.name+"-->"+bindingName);
        
        Binding bindingEntry = null; 
        Iterator bindings =  wsdlDocument.getBindings().values().iterator();
        foundIt = false;
        
        while ((bindings.hasNext())&&(!foundIt)){
            bindingEntry = (Binding) bindings.next();
//            foundIt = bindingEntry.getQName().getLocalPart().equals(bindingName);
            foundIt = bindingEntry.getQName().getLocalPart().equalsIgnoreCase(bindingName);
        }
        
        if (foundIt){
            PortType portTypeEntry = bindingEntry.getPortType();

            System.out.println("Port Name --> " + portTypeEntry.getQName().getLocalPart());

            List operationList = portTypeEntry.getOperations();
            Iterator it = operationList.iterator();
            Operation operation;
            
            while (it.hasNext()) {
                operation = (Operation) it.next();
                methodName = operation.getName();
                System.out.println("Operation Name --> " + methodName );
                methodList.add(new MethodCodeProcessor(methodName, targetMethodsPrefix));

            }
        }
                
       return methodList;
    }
            
    private String contructorPocessor (String code){
        String localCode = code;

        String constructorRegExpr = "public " + this.name + "\\s*\\(\\s*\\)\\s*\\{"; 

        Pattern constructorPattern = Pattern.compile(constructorRegExpr, Pattern.MULTILINE);
        Matcher constructorDefinitionMatcher = constructorPattern.matcher(code); 

        String constructorDefinitionString;
        if (constructorDefinitionMatcher.find()){
/*
* The class stored the code variable has the default constructor.  
*/            
            constructorDefinitionString = constructorDefinitionMatcher.group();
/*
 * Adding possible throws statements
 */            
            constructorDefinitionString = constructorDefinitionString.substring(0, constructorDefinitionString.length()-1) + this.ambitionPatterns.getExceptionStatements() + " {";
            
            int start = constructorDefinitionMatcher.start();
            int offset = constructorDefinitionMatcher.end();
                                
//          The attributes for the integration with Ambition are plugged in the class code.
            String tempBuffer = ambitionPatterns.getDeclarationItems() + constructorDefinitionString;
//          The default constructor with the code for the integration with Ambition is plugged in the class code.
            tempBuffer += this.ambitionPatterns.getSimulatorInstantiationCode();

//          The new version of the class is generated;            
            localCode = this.exchangePortion(code, start, offset, tempBuffer);
        }
        else{
/*
 * The class stored the code variable does not have any default constructor.  
 */            
            String classRegExpr = "public class " + this.name + "[^{]*\\{";

            Pattern classPattern = Pattern.compile(classRegExpr, Pattern.MULTILINE);
            Matcher classDefinitionMatcher = classPattern.matcher(code); 

            if (classDefinitionMatcher.find()){
                String classDefinitionString = classDefinitionMatcher.group();
            
                int start = classDefinitionMatcher.start();
                int offset = classDefinitionMatcher.end();
                                
//          The attributes for the integration with Ambition are plugged in the class code.
                String tempBuffer = classDefinitionString + ambitionPatterns.getDeclarationItems();
//          The default constructor with the code for the integration with Ambition is plugged in the class code.
                tempBuffer += "\n public " + this.name + "()" + this.ambitionPatterns.getExceptionStatements() + " { " + this.ambitionPatterns.getSimulatorInstantiationCode() + "}";
 
// The new version of the class is generated;            
                localCode = this.exchangePortion(code, start, offset, tempBuffer);
            }
        }
        
        StringTemplate templateParser = new StringTemplate(localCode);
        templateParser.setAttribute("ambitionPrefix", ambitionVarWrapper.getAmbitionPrefix());        
        templateParser.setAttribute("WSDLUrl", Messages.WSDL_URL_MESSAGE);        
        templateParser.setAttribute("WSDLService", Messages.WSDL_SERVICE_MESSAGE);        
        templateParser.setAttribute("WSDLPort", Messages.WSDL_PORT_MESSAGE); 
        templateParser.setAttribute("solverHost", Messages.SOLVER_HOST_MESSAGE);        
        templateParser.setAttribute("solverPort", Messages.SOLVER_PORT_MESSAGE); 
        templateParser.setAttribute("SSMSchemaInstance", Messages.SSM_SCHEMA_INSTANCE_URL_MESSAGE); 
        
        localCode = templateParser.toString();
        
        
        return localCode;
    }
    
    public String parseClass (String code){
        String localCode = code;

        ArrayList<MethodCodeProcessor> methodsList = extractMethodsNameOnly(localCode);

        for (int i=0; i<methodsList.size();i++){
            localCode = methodsList.get(i).parseCode(localCode);
        }
                    
        localCode = this.contructorPocessor(localCode);
        
        return localCode; 
    }
    
    private String exchangePortion(String originalString, int beginRegion, int endRegion, String newString){

        int originalStringLength =  originalString.length();
        String exchangedString = originalString.substring(0,beginRegion)+newString+originalString.substring(endRegion, originalStringLength);            
        return exchangedString;
    }
        
}
