package puppet.ambition;

public class PatternsReader {
    
    private String invocationTimeVarName = "invocationTime";
    private String returnValueVarName = "returnValue";
    private TypeConverter converter;
    
    public PatternsReader() {
        converter = new TypeConverter();
    }
        
    public String getExceptionStatements (){
       return " throws Exception ";    
    }
    
    public String getCommonVariableCode(String methodReturnType){
        String varCode;
//      that's a bug!! the \\ are required since it is invoked by a String.repalceFirst 
//      method whose will crash without them because of the $s        
         varCode = "long \\$ambitionPrefix\\$" + invocationTimeVarName + " = 0; \n"; 
         if (!methodReturnType.equals("void"))
             varCode +=  methodReturnType + " \\$ambitionPrefix\\$"+returnValueVarName+"; \n";
        return varCode; 
    }
    
    public String castExceptions(String codeToCast, String exceptionName){
        return "try { \n " + codeToCast + " \n } catch (Exception genericException ){ \n throw new "+ exceptionName +"(genericException.getMessage()); \n } \n";
    }

    public String getElapsedTimeStatement(){
        return " - puppet.ambition.Naturals.asNatural(\\$ambitionPrefix\\$"+invocationTimeVarName+" - System.currentTimeMillis())";
    }
    
    public String getCurrentTimeStampStatements (){
        
//  that's a bug!! the \\ are required since it is invoked by a String.repalceFirst 
//        method whose will crash without them because of the $s        
//        return "long \\$ambitionPrefix\\$" + invocationTimeVarName + " = System.currentTimeMillis(); ";        
        return "\\$ambitionPrefix\\$" + invocationTimeVarName + " = System.currentTimeMillis(); ";        
    }
    
    public String getDeclarationItems(){
        
        String code="";       
                
        // The SSM object.
        code +="/* The SSM object. \n */ \n private info.frantzen.testing.ssmsimulator.ssm.ServiceStateMachine $ambitionPrefix$ssm; \n";
        
        // The simulator object
        code +="/* The simulator is generated \n */ \n private info.frantzen.testing.ssmsimulator.SSMSimulator $ambitionPrefix$sim ;\n";
        
        // A definition of a local method
        code += "private info.frantzen.testing.ssmsimulator.ssm.Message $ambitionPrefix$findSSMMessage(info.frantzen.testing.ssmsimulator.ssm.ServiceStateMachine ssm, info.frantzen.testing.ssmsimulator.ssm.MessageKind kind, info.frantzen.testing.ssmsimulator.ssm.Operation op) throws Exception { " +
                  "java.util.HashSet<info.frantzen.testing.ssmsimulator.ssm.Message> messages = ssm.getMessages(); " +
                  "for (java.util.Iterator it = messages.iterator(); it.hasNext(); ) { " +
                    "info.frantzen.testing.ssmsimulator.ssm.Message m = (info.frantzen.testing.ssmsimulator.ssm.Message) it.next(); " +
                    "if (m.getKind() != info.frantzen.testing.ssmsimulator.ssm.MessageKind.UNOBSERVABLE) { " +
                      "if ((m.getKind() == kind) && (m.getOperation().equals(op))) { " +
                         "return m; " +
                    "}}} " +
                "throw new Exception(\"Cannot find the input SSM message belonging to the Operation \" + op.getName()+ \"!\");}"; 

        return code;
    }
    
    public String getSimulatorInstantiationCode(){
        
        String code = "\n   /* To initialise the Simulator, the following items are needed: \n */ \n";       
        
        // The URL of the WSDL file
        code += "\n   /* The URL of the WSDL file \n */ \n java.net.URL $ambitionPrefix$WSDLUrl = new java.net.URL(\"$WSDLUrl$\"); \n";
        
        // The name of the WSDL-Service
        code +="\n   /* The name of the WSDL-Service \n */ \n String $ambitionPrefix$service = \"$WSDLService$\";\n";
        
        // The name of the WSDL-Port
        code +="\n   /* The name of the WSDL-Port \n */ \n String $ambitionPrefix$port = \"$WSDLPort$\";\n";
        
        // The URL of the SSM Schema instance
        code +="\n   /* The URL of the SSM Schema instance \n */ \n java.net.URL $ambitionPrefix$SSMUrl = new java.net.URL(\"$SSMSchemaInstance$\");\n";
        
//        // Now we can generate the SSM object. To do so, we use Zsolt's "Minerva" library
//      code +="/* Now we can generate the SSM object. To do so, we use Zsolt's \"Minerva\" library \n */ \n info.frantzen.testing.ssmsimulator.ssm.ServiceStateMachine $ambitionPrefix$ssm = hu.soft4d.jessi.ssm.SSMHandler.generateSSM($ambitionPrefix$WSDLUrl, $ambitionPrefix$SSMUrl, $ambitionPrefix$service, $ambitionPrefix$port);\n";
        // Now we can generate the SSM object. To do so, we use Zsolt's "Minerva" library
       code +="\n   /* Now we can generate the SSM object. To do so, we use Zsolt's \"Minerva\" library \n */ \n $ambitionPrefix$ssm = hu.soft4d.jessi.ssm.SSMHandler.generateSSM($ambitionPrefix$WSDLUrl, $ambitionPrefix$SSMUrl, $ambitionPrefix$service, $ambitionPrefix$port);\n";
        
        // Before we can use the SSM in the simulator, the parsers have to be attached to the switches
        code +="\n   /* Before we can use the SSM in the simulator, the parsers have to be attached to the switches \n */ \n $ambitionPrefix$ssm.attachParsersToSwitches();\n";
        
        // Next we generate the socket to the treeSolver.
        code +="\n   /* Next we generate the socket to the treeSolver. \n */ \n String $ambitionPrefix$solverHost = \"$solverHost$\"; \n int $ambitionPrefix$solverPort = $solverPort$; \n java.net.Socket $ambitionPrefix$solverSocket = new java.net.Socket($ambitionPrefix$solverHost, $ambitionPrefix$solverPort);\n";
        
        // The treeSolver sends a welcome message, we remove it from the stream
        code +="\n   /* The treeSolver sends a welcome message, we remove it from the stream \n */ \n new java.io.BufferedReader(new java.io.InputStreamReader($ambitionPrefix$solverSocket.getInputStream())).readLine();\n";
        
        code +="\n   /* The simulator can use an external tool to display sequence diagrams of the messages exchanged. \n // I skip this here since this takes extra ressources/ \n */ \n";
        
        // The simulator needs a logger to log to
        code +="\n   /* The simulator needs a logger to log to \n */ \n java.util.logging.Logger $ambitionPrefix$logger = java.util.logging.Logger.getLogger(\"$LoggerName$\");\n";
        
//        // The simulator is generated
//        code +="/* The simulator is generated \n */ \n info.frantzen.testing.ssmsimulator.SSMSimulator $ambitionPrefix$sim = new info.frantzen.testing.ssmsimulator.SSMSimulator($ambitionPrefix$ssm, $ambitionPrefix$solverSocket, $ambitionPrefix$logger);\n";
        // The simulator is generated
        code +="\n   /* The simulator is generated \n */ \n $ambitionPrefix$sim = new info.frantzen.testing.ssmsimulator.SSMSimulator($ambitionPrefix$ssm, $ambitionPrefix$solverSocket, $ambitionPrefix$logger);\n";
        
        // If Double variables are used we assume this models money (experimental). In any case, do this:
        code += "\n   /* If Double variables are used we assume this models money (experimental). In any case, do this: \n */ \n info.frantzen.testing.ssmsimulator.types.ST_PseudoPosDouble.postPointLength = 2; \n";
        
        // Now the Simulator is ready. --------------------------------------------------------------------------
        code +="\n   /* Now the Simulator is ready. --------------------------------------------------------------------------\n */ \n";
        
        return code;
    }
    
    public String getMessageSelectionCode(boolean isOneWay){
        String code;
        if (isOneWay) 
            code = "info.frantzen.testing.ssmsimulator.ssm.Operation $ambitionPrefix$operation = new info.frantzen.testing.ssmsimulator.ssm.Operation(\"$operationName$\", info.frantzen.testing.ssmsimulator.ssm.OperationKind.ONEWAY); \n";
        else
            code = "info.frantzen.testing.ssmsimulator.ssm.Operation $ambitionPrefix$operation = new info.frantzen.testing.ssmsimulator.ssm.Operation(\"$operationName$\", info.frantzen.testing.ssmsimulator.ssm.OperationKind.REQUESTRESPONSE); \n";
        code += "info.frantzen.testing.ssmsimulator.ssm.Message $ambitionPrefix$message = $ambitionPrefix$findSSMMessage($ambitionPrefix$ssm, info.frantzen.testing.ssmsimulator.ssm.MessageKind.INPUT, $ambitionPrefix$operation);\n" ;
        code += "info.frantzen.testing.ssmsimulator.ssm.Valuation $ambitionPrefix$valuation = new info.frantzen.testing.ssmsimulator.ssm.Valuation(); \n " ;
        code += "java.util.ArrayList<info.frantzen.testing.ssmsimulator.ssm.InteractionVariable> $ambitionPrefix$messageType = $ambitionPrefix$message.getType(); \n";
        code += "java.util.Iterator $ambitionPrefix$it = $ambitionPrefix$messageType.iterator();\n";
        code += "info.frantzen.testing.ssmsimulator.ssm.InteractionVariable $ambitionPrefix$var;\n";
        return code;
                
    }
        
    public String getSimulatorInvocationCode(){
        String code;
        
        code = "/* The valuation is ready, we can construct an instantiated message */ \n";
        code += "info.frantzen.testing.ssmsimulator.ssm.InstantiatedMessage $ambitionPrefix$im = new info.frantzen.testing.ssmsimulator.ssm.InstantiatedMessage($ambitionPrefix$message, $ambitionPrefix$valuation);\n";
        
        code += "\n /* This instantiated message can now be given to the simulator. Note that here the simulator can potentially spot a failure, namely when this message is not specified in the SSM! In that sense, here we do testing.*/ \n";
        code += "$ambitionPrefix$sim.processInstantiatedMessage($ambitionPrefix$im);\n";
        
        return code;
    }
    
    public String getSimulatorInvocationCode(String returnType){
        String code = getSimulatorInvocationCode(); 
            
// This part is used in order to select one among the results        
        code = commonSimulatorReturnCode(code);

        code += " $ambitionPrefix$"+ returnValueVarName +" = ("+converter.getNameObjectType(returnType)+")$ambitionPrefix$returnInstance.getValue();\n";        
        code += "\n /* Now send the returnValue back to the calling service. That's it.*/ \n";
        
        return code;
    }

/*
 * ATTENTION: after this invokation you would write the code in order to cast the ComplexType
 */
    public String getSimulatorInvocationCodeComplexTypes (){
        String code = getSimulatorInvocationCode(); 
        
//      This part is used in order to select one among the results        
             code = commonSimulatorReturnCode(code);
        return code;     
   
    }
    
    private String commonSimulatorReturnCode(String code) {
        code += "\n /* Ok, the simulator knows the input. Now we need a functionally correct response to this call. We first ask the simulator for all currently activated output transitions.*/ \n";
        code += " java.util.ArrayList $ambitionPrefix$outputs = new java.util.ArrayList($ambitionPrefix$sim.getCurrentOutputSwitches());\n";
        
        code += "\n /* Out of all possible output switches, we randomly choose one and check if it has a solution. If yes, we take it. If not, we choose randomly the next one.*/ \n";
        code += " boolean $ambitionPrefix$noSolutionFound = true;\n";
        code += " info.frantzen.testing.ssmsimulator.ssm.InstantiatedMessage $ambitionPrefix$nextOutput = null;\n";
        code += " java.util.Random $ambitionPrefix$random = new java.util.Random();\n";
        code += " while (!$ambitionPrefix$outputs.isEmpty() && $ambitionPrefix$noSolutionFound) {";
        code += "  info.frantzen.testing.ssmsimulator.ssm.Switch $ambitionPrefix$candidate = (info.frantzen.testing.ssmsimulator.ssm.Switch) $ambitionPrefix$outputs.get($ambitionPrefix$random.nextInt($ambitionPrefix$outputs.size()));\n";
        code += "  \n /* try to find a solution, if yes, fine, if not, remove the candidate */ \n";
        code += "  $ambitionPrefix$nextOutput = $ambitionPrefix$sim.findSolution($ambitionPrefix$candidate);\n";
        code += "    if ($ambitionPrefix$nextOutput == null)";
        code += "        $ambitionPrefix$outputs.remove($ambitionPrefix$candidate);\n";
        code += "    else $ambitionPrefix$noSolutionFound = false;\n";
        code += " }";
// That should be a bug from Lars
//        code += " if ($ambitionPrefix$nextOutput != null)";
        code += " if ($ambitionPrefix$nextOutput == null)";
        code += "    throw new Exception(\"Failure in SSM! No output for synchronous input specified!\");\n";
        
        code += "\n /* Ok, we have now a feasible and functionally correct output: nextOutput \n Before we send this output to the Service out there, we tell so to the simulator:*/ \n";
// That step would corrupt the backup sistem of an opreation request on the simulator
//        code += " $ambitionPrefix$sim.processInstantiatedMessage($ambitionPrefix$nextOutput);";
        code += " $ambitionPrefix$sim.processInstantiatedMessageNoBackup($ambitionPrefix$nextOutput);";

        code += "\n /* What is left to do, is to map this instantiated message back to a real returnValue.*/ \n";
        code += " info.frantzen.testing.ssmsimulator.ssm.Message $ambitionPrefix$returnMessage = $ambitionPrefix$nextOutput.getMessage();\n";
        code += " String $ambitionPrefix$returnVarName = ((info.frantzen.testing.ssmsimulator.ssm.InteractionVariable) $ambitionPrefix$returnMessage.getType().iterator().next()).getName();\n";
        code += " info.frantzen.testing.ssmsimulator.ssm.Valuation $ambitionPrefix$returnValuation = $ambitionPrefix$nextOutput.getValuation();\n";
        code += " info.frantzen.testing.ssmsimulator.types.TypeInstance $ambitionPrefix$returnInstance = $ambitionPrefix$returnValuation.getSingleInstance($ambitionPrefix$returnVarName);\n";        
        
        return code;
    }
        
    public String getReturnCode (){
        return "return $ambitionPrefix$returnValue;";
    }

    public String getReturnValueVarName() {
        return returnValueVarName;
    }
    
    public String castStatementUsingAmbitionTypes (String inStatement, String castingType){
        TypeConverter converter = new TypeConverter();
        String basicType = converter.getAmbitionType2JavaBasicType(castingType);
        if (basicType.equals("int"))
            return "(Integer.valueOf("+inStatement+").intValue())";
        if (basicType.equals("double"))
            return "(Double.valueOf("+inStatement+").doubleValue())";
        if (basicType.equals("boolean"))
            return "(Boolean.valueOf("+inStatement+").booleanValue())";
        if (basicType.equals("Enumeration"))
            return "((Enum) " + inStatement +")";
        return inStatement;
    }

    public String castStatement (String inStatement, String castingType){
        if (castingType.equals("int"))
            return "(Integer.valueOf("+inStatement+").intValue())";
        if (castingType.equals("double"))
            return "(Double.valueOf("+inStatement+").doubleValue())";
        if (castingType.equals("boolean"))
            return "(Boolean.valueOf("+inStatement+").booleanValue())";
        if (castingType.equals("Enumeration"))
            return "((Enum) " + inStatement +")";
        return inStatement;
    }
}
