package puppet.mobility.oracle;

import java.io.BufferedReader;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;

public class TraceReaderThread extends Thread{

    protected BufferedReader traceFile;
    protected AbstractTraceReader traceReader;
    protected ArrayList<SimulatedEvent> events;   
    protected boolean stopTheExecution;
    
    public TraceReaderThread (AbstractTraceReader traceReader, BufferedReader traceFile){
        this.traceFile = traceFile;
        this.stopTheExecution = false;
        this.traceReader = traceReader;
        this.events = new ArrayList<SimulatedEvent>();
    }
    
    public void run() {
       long timeToRefresh = 0;
       long lastUpdateSimulationTime = 0;
       long currentUpdateSimulationTime = 0;
       while (!stopTheExecution){
           lastUpdateSimulationTime = currentUpdateSimulationTime; 
           currentUpdateSimulationTime = updatePositions();
           if (currentUpdateSimulationTime == Constants4Mobility.NO_MORE_UPDATE_POSSIBLE_CODE)
               this.stopThread();
           else{
               timeToRefresh = Math.abs(currentUpdateSimulationTime - lastUpdateSimulationTime);
               traceReader.setEventList(events);
               try {
//                   System.out.println("Next refresh in "+ timeToRefresh + " msecs");
                   Thread.sleep(timeToRefresh);
               } catch (InterruptedException e) {}
           }
       }           
    }
    
    /*
     * Returns Constants4Mobility.NO_MORE_UPDATE_POSSIBLE_CODE (conv. -1) if the end of the file is reached and no more updates are possible.
     */
    protected long updatePositions (){
        String currentLine;
        SimulatedEvent currentEvent;
        long simulationTimeAtUpdate = 0; 
        int index;
        boolean wasTheEventListCleaned = false;
        
        try {
            currentLine = this.traceFile.readLine();
            if (currentLine == null)
                return Constants4Mobility.NO_MORE_UPDATE_POSSIBLE_CODE;
//            System.out.println("Upating Positions ...");        
            index = 0;
//            while ( (index < Constants4Mobility.NODES) && (currentLine != null)){
            while ( (index < traceReader.nodesNumber) && (currentLine != null)){
                try {
                    currentEvent = new SimulatedEvent(currentLine);
                    if (currentEvent.getEvent().compareTo(EventsType.SEND) == 0){
//System.out.println(currentEvent.getNodeName() + " updated"); 
                        if (!wasTheEventListCleaned){
                            this.events.clear();
                            wasTheEventListCleaned = true;
                        }
                        this.events.add(currentEvent);
                        index++;
                    // The Simulaltion Time in SimulatedEvents is expressed in Seconds
                        simulationTimeAtUpdate = Math.round(currentEvent.getSimulationTime()*1000);
                    }    
                } catch (ParseException e) { 
//                  There is nothing to do here!!!                    
                }
                currentLine = this.traceFile.readLine();
            }            
        } catch (IOException e) {
            e.printStackTrace();
        }
//System.out.println("Positions Updated");  
        return simulationTimeAtUpdate;
    }
    
    public void stopThread(){
        stopTheExecution = true;
    }
}
