package puppet.slang;

import java.io.File;
import java.io.IOException;


import org.ggf.schemas.graap.x2005.x09.wsAgreement.TermCompositorType;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.AgreementOfferDocument;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.AgreementType;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.TermTreeType;

import uk.ac.ucl.cs.slangta.jmi.impl.slang.SLA_Impl;
import uk.ac.ucl.cs.slangta.jmi.slang.es.FixedLatencyFailureModeDefinition;
import uk.ac.ucl.cs.slangta.jmi.slang.es.InputThroughputScheduledSlidingPenaltyClause;
import uk.ac.ucl.cs.slangta.jmi.slang.es.ReliabilityFixedWindowFixedFailuresScheduledSlidingPenaltyClause;

public class AgreementGenerator {

    private AgreementOfferDocument aod;

//    private SLA_Impl slangSpecification;
    private SLAngPuppetSpecification slangSpecification;
    
    private String targetDir;
    private String targetFile;

    private LatencyGuaranteeTermsGenerator latencyGenerator;
    private ReliabilityGuaranteeTermsGenerator reliabilityGenerator;
    private WorkloadGuaranteeTermsGenerator workloadGenerator;

    public AgreementGenerator(SLAngPuppetSpecification slangSpecification) {
        this(slangSpecification,"./","out.wsa");
    }

    public AgreementGenerator(SLAngPuppetSpecification slangSpecification, String targetFile) {
        this(slangSpecification,"./",targetFile);
    }

    public AgreementGenerator(SLAngPuppetSpecification slangSpecification, String targetDir, String targetFile) {
        aod = this.generateCommonStructure();
        this.targetDir = targetDir;
        this.targetFile = targetFile;
        this.slangSpecification = slangSpecification;
    }

    public AgreementOfferDocument generateCommonStructure() {
        AgreementOfferDocument aod = AgreementOfferDocument.Factory
                .newInstance();
        AgreementType at = aod.addNewAgreementOffer();
        at.addNewContext();
        at.addNewTerms();
        return aod;
    }

    public TermTreeType generateSLATerms() {

        TermCompositorType termReference;
        TermTreeType terms = aod.getAgreementOffer().getTerms();

        terms.addNewAll();
        termReference = terms.getAll();

        java.util.Collection value = slangSpecification.getSlangSpecification().getAuxiliaryClauses();
		
		if(value != null) {
		
			javax.jmi.reflect.RefObject [] values =
				(javax.jmi.reflect.RefObject [])value.toArray(
					new javax.jmi.reflect.RefObject[value.size()]);
			for(int n = 0; n < values.length; n++) {
				
				String objName = values[n].getClass().getName();
				
				if ( objName == "uk.ac.ucl.cs.slangta.jmi.impl.slang.es.InputThroughputScheduledSlidingPenaltyClause_Impl") {
					// an InputThroughput clause = workload					
					InputThroughputScheduledSlidingPenaltyClause itc = (InputThroughputScheduledSlidingPenaltyClause) values[n];
					// Max n. of requests:
					int maxnreq = itc.getConcurrency();
					// Window in milliseconds
					double itcwindow = itc.getWindow().inMs();
					
			        workloadGenerator = new WorkloadGuaranteeTermsGenerator(slangSpecification.getSlangSpecification(),maxnreq,itcwindow,termReference,slangSpecification.getServiceURL(),slangSpecification.getClientServiceName());        
			        workloadGenerator.generate();
					
				}
				
				if ( objName == "uk.ac.ucl.cs.slangta.jmi.impl.slang.es.FixedLatencyFailureModeDefinition_Impl" ) {
					// Latency
					
					FixedLatencyFailureModeDefinition latency = (FixedLatencyFailureModeDefinition)values[n];

					// Latency in ms
					double latencyms = latency.getMaxDuration().inMs();

					latencyGenerator = new LatencyGuaranteeTermsGenerator(slangSpecification.getSlangSpecification(),latencyms,termReference);
			        latencyGenerator.generate();
				
				}

				if ( objName == "uk.ac.ucl.cs.slangta.jmi.impl.slang.es.ReliabilityFixedWindowFixedFailuresScheduledSlidingPenaltyClause_Impl") {
					// Reliability
					
					ReliabilityFixedWindowFixedFailuresScheduledSlidingPenaltyClause rel = (ReliabilityFixedWindowFixedFailuresScheduledSlidingPenaltyClause)values[n];
					
					// the max number of failures...
					int maxfailures = rel.getMaxFailures();

					// ... in this time window:
					
					double relwindow = rel.getReliabilityWindow().inMs();

			        reliabilityGenerator = new ReliabilityGuaranteeTermsGenerator(slangSpecification.getSlangSpecification(),maxfailures,relwindow,termReference);
			        reliabilityGenerator.generate();
				
				}
			
			}
		}
		        
        return terms;
        
    }
    
    public void generate(){
        String absoluteFileName = this.targetDir + "/" + this.targetFile;
        
        this.aod = this.generateCommonStructure();   
        this.generateSLATerms();
        try {
            this.aod.save(new File(absoluteFileName));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public String getTargetDir() {
        return targetDir;
    }

    public void setTargetDir(String targetDir) {
        this.targetDir = targetDir;
    }

    public String getTargetFile() {
        return targetFile;
    }

    public void setTargetFile(String targetFile) {
        this.targetFile = targetFile;
    }

    public void setSLA(SLAngPuppetSpecification sla) {
    	slangSpecification	= sla;
    }
        
}
